import 'package:diameter/components/progress_indicator.dart';
import 'package:diameter/models/log_event_type.dart';
import 'package:diameter/navigation.dart';
import 'package:diameter/screens/log/log_event_type_detail.dart';
import 'package:flutter/material.dart';

class LogEventTypeListScreen extends StatefulWidget {
  static const String routeName = '/log-event-types';
  const LogEventTypeListScreen({Key? key}) : super(key: key);

  @override
  _LogEventTypeListScreenState createState() => _LogEventTypeListScreenState();
}

class _LogEventTypeListScreenState extends State<LogEventTypeListScreen> {
  late Future<List<LogEventType>?> _logEventTypes;

  void refresh({String? message}) {
    setState(() {
      _logEventTypes = LogEventType.fetchAll();
    });
    setState(() {
      if (message != null) {
        var snackBar = SnackBar(
          content: Text(message),
          duration: const Duration(seconds: 2),
        );
        ScaffoldMessenger.of(context)
          ..removeCurrentSnackBar()
          ..showSnackBar(snackBar);
      }
    });
  }

  @override
  void initState() {
    super.initState();
    refresh();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(title: const Text('Log Event Types'), actions: <Widget>[
        IconButton(onPressed: refresh, icon: const Icon(Icons.refresh))
      ]),
      drawer:
          const Navigation(currentLocation: LogEventTypeListScreen.routeName),
      body: Column(
        mainAxisAlignment: MainAxisAlignment.center,
        children: <Widget>[
          Expanded(
            child: FutureBuilder<List<LogEventType>?>(
              future: _logEventTypes,
              builder: (context, snapshot) {
                return ViewWithProgressIndicator(
                  snapshot: snapshot,
                  child: snapshot.data == null || snapshot.data!.isEmpty
                      ? Row(
                          mainAxisAlignment: MainAxisAlignment.center,
                          children: const [
                            Padding(
                              padding: EdgeInsets.all(10.0),
                              child: Text('No Log Event Types'),
                            ),
                          ],
                        )
                      : ListView.builder(
                          padding: const EdgeInsets.all(10.0),
                          itemCount:
                              snapshot.data != null ? snapshot.data!.length : 0,
                          itemBuilder: (context, index) {
                            final logEventType = snapshot.data![index];
                            return ListTile(
                              onTap: () {
                                Navigator.push(
                                  context,
                                  MaterialPageRoute(
                                    builder: (context) =>
                                        LogEventTypeDetailScreen(
                                            logEventType: logEventType),
                                  ),
                                ).then((message) => refresh(message: message));
                              },
                              title: Text(logEventType.value),
                              subtitle: Text(logEventType.notes ?? ''),
                              trailing: Row(
                                mainAxisSize: MainAxisSize.min,
                                children: [
                                  IconButton(
                                    onPressed: () async {
                                      await logEventType.delete().then((_) {
                                        refresh(
                                            message: 'Log Event Type deleted');
                                      });
                                    },
                                    icon: const Icon(Icons.delete,
                                        color: Colors.blue),
                                  )
                                ],
                              ),
                            );
                          },
                        ),
                );
              },
            ),
          ),
        ],
      ),
      floatingActionButton: FloatingActionButton(
        onPressed: () {
          Navigator.push(
            context,
            MaterialPageRoute(
              builder: (context) => const LogEventTypeDetailScreen(),
            ),
          ).then((message) => refresh(message: message));
        },
        child: const Icon(Icons.add),
      ),
    );
  }
}
